/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/

package org.fortiss.af3.rcp.application.handler;

import static org.conqat.ide.commons.ui.logging.LoggingUtils.logAndShowError;
import static org.eclipse.emf.common.util.URI.createFileURI;
import static org.eclipse.emf.common.util.URI.createPlatformPluginURI;
import static org.eclipse.jface.dialogs.ErrorDialog.openError;
import static org.eclipse.ui.PlatformUI.getWorkbench;
import static org.eclipse.ui.internal.about.AboutUtils.openErrorLogBrowser;
import static org.fortiss.af3.project.AF3Project.AF3_PRJ_SUFFIX;
import static org.fortiss.af3.project.ui.utils.ExportProjectUtils.exportProject;
import static org.fortiss.af3.project.ui.utils.ExportProjectUtils.fileSelectionDialog;
import static org.fortiss.af3.project.ui.utils.ExportProjectUtils.getSelectedFileProject;
import static org.fortiss.af3.project.ui.utils.ImportProjectUtils.fileSelectionDialog;
import static org.fortiss.af3.project.ui.utils.ImportProjectUtils.importUISelectedFile;
import static org.fortiss.af3.rcp.application.utils.ApplicationUtils.switchToAF3Perspective;
import static org.fortiss.tooling.kernel.utils.EMFResourceUtils.refreshWorkspace;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.copy;

import java.io.File;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.swt.widgets.Display;
import org.fortiss.af3.expression.examples.ConfiguredExpressionExample;
import org.fortiss.af3.mode.AF3ModeActivator;
import org.fortiss.af3.mode.ui.examples.SimpleTrafficLightsModesExample;
import org.fortiss.af3.project.example.IExample;
import org.fortiss.af3.project.model.FileProject;
import org.fortiss.af3.rcp.application.AF3ApplicationActivator;
import org.fortiss.af3.rcp.application.example.SimpleTrafficLightsCodeGenerationExample;
import org.fortiss.af3.rcp.application.utils.ApplicationUtils;
import org.fortiss.tooling.kernel.extension.data.ModelStorageError;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * The ApplicationCommandHandler has several nested classes, each for handling a
 * different command action
 * 
 * @author becker
 * @author wandinger
 */

public class ApplicationCommandHandler {

	/** Creates a project (per code) and stores it. */
	private static boolean createAndSaveProject(String projectName, IExample builder) {
		return ApplicationUtils.createAndSaveProject(projectName, AF3_PRJ_SUFFIX, builder);
	}

	/**
	 * Loads an existing *.af3_2x Example file from a given path and adds it to
	 * the Model-Navigator.
	 * 
	 * @param pluginID
	 *            the plugin ID
	 * @param relativePath
	 *            the relative path to the file
	 * @param projectName
	 *            the name of the file
	 */
	private synchronized static void loadExistingModel(String pluginID, String relativePath,
			String projectName) {
		URI fileURI = createPlatformPluginURI(pluginID + File.separator + relativePath, true);

		try {
			importUISelectedFile(fileURI);
		} catch(Exception e) {
			logAndShowError(AF3ApplicationActivator.getDefault(), "The project cannot be loaded.",
					e);
		}
	}

	/** Command to exit the application. */
	public static final class AF3ExitCommandHandler extends AbstractHandler {

		/** {@inheritDoc} */
		@Override
		public Object execute(ExecutionEvent event) {
			getWorkbench().close();
			return null;
		}
	}

	/** Handles the command action to create a new AF3 project. */
	public static final class NewAF3ProjectCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that creates a new AF3 project and, if required, a
		 * new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			// Ensure existence of generic project and enable AF3 Perspective.
			switchToAF3Perspective();

			// Create a new AF3 project.
			createAndSaveProject("AF3-Project", ConfiguredExpressionExample.INSTANCE);

			return null;
		}
	}

	/** Handles the command action to create the standard SimpleTrafficLightsExample project. */
	public static final class LoadSimpleTrafficLightsExampleCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that creates the SimpleTrafficLightsExample
		 * project and, if required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ApplicationActivator.PLUGIN_ID,
					"test-data/SimpleTrafficLightsExample.af3_23", "SimpleTrafficLightsExample");
			return null;
		}
	}

	/**
	 * Handles the command action to create the SimpleTrafficLightsExample
	 * project that introduces modes.
	 */
	public static final class LoadSimpleTrafficLightsModesExampleCommandHandler
			extends AbstractHandler {

		/**
		 * Execute definition that creates the SimpleTrafficLightsExample
		 * project and, if required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			// Ensure existence of generic project and enable AF3 Perspective.
			switchToAF3Perspective();

			// Load SimpleTrafficLightsExample.
			createAndSaveProject("SimpleTrafficLightsModesExample",
					SimpleTrafficLightsModesExample.INSTANCE);

			return null;
		}
	}

	/**
	 * Handles the command action to create a version of the
	 * SimpleTrafficLightsExample which contains a well defined set of
	 * components, that can be generated.
	 */
	public static final class LoadSimpleTrafficLightsCodeGenerationExampleCommandHandler
			extends AbstractHandler {

		/**
		 * Execute definition that creates the SimpleTrafficLightsExample
		 * project and, if required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			// Ensure existence of generic project and enable AF3 Perspective.
			switchToAF3Perspective();

			// Load SimpleTrafficLightsCodeGenerationExample.
			createAndSaveProject("SimpleTrafficLightsCodeGenerationExample",
					SimpleTrafficLightsCodeGenerationExample.INSTANCE);

			return null;
		}
	}

	/** Handles the command action to create the SimpleTrafficLights_AF3Tour project. */
	public static final class LoadSimpleTrafficLights_AF3TourCommandHandler
			extends AbstractHandler {

		/**
		 * Execute definition that creates the SimpleTrafficLights_AF3Tour-Example project
		 * and, if required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ApplicationActivator.PLUGIN_ID,
					"test-data/SimpleTrafficLights_AF3Tour.af3_23", "SimpleTrafficLights_AF3Tour");
			return null;
		}
	}

	/** Handles the command action to create the MotorManagement-Example project. */
	public static final class LoadMotorManagementExampleCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that creates the MotorManagement-Example project
		 * and, if required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ModeActivator.PLUGIN_ID, "test-data/MotorManagement.af3_23",
					"MotorManagement");
			return null;
		}
	}

	/** Handles the command action to create the ACC-Example project. */
	public static final class LoadACCExampleCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that creates the ACC-Example project and, if
		 * required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ApplicationActivator.PLUGIN_ID, "test-data/ACC.af3_23", "ACC");
			return null;
		}
	}

	/** Handles the command action to create the ACC-Refinement-Example project. */
	public static final class LoadACCRefinementExampleCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that creates the ACC-Refinement-Example project
		 * and, if required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ApplicationActivator.PLUGIN_ID, "test-data/ACCRefinement.af3_23",
					"ACCRefinementExample");
			return null;
		}
	}

	/** Handles the command action to create the eCar-Example project. */
	public static final class LoadECarExampleCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that creates the eCar-Example project and, if
		 * required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ApplicationActivator.PLUGIN_ID, "test-data/eCar.af3_23", "eCar");
			return null;
		}
	}

	/** Handles the command action to create the EmergencyStop-Example project. */
	public static final class LoadEmergencyStopExampleCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that creates the EmergencyStop-Example project
		 * and, if required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ApplicationActivator.PLUGIN_ID,
					"test-data/Emergency-Stop-Module.af3_23", "Emergency-Stop-Module");
			return null;
		}
	}

	/** Handles the command action to create the EmergencyStopWeaklyCausal-Example project. */
	public static final class LoadEmergencyStopWeaklyCausalExampleCommandHandler
			extends AbstractHandler {

		/**
		 * Execute definition that creates the EmergencyStop-Example project
		 * and, if required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ApplicationActivator.PLUGIN_ID,
					"test-data/Emergency-Stop-Module-Weakly-Causal.af3_23",
					"Emergency-Stop-Module-Weakly-Causal");
			return null;
		}
	}

	/** Handles the command action to create the Operator-Panel-Example project. */
	public static final class LoadOperatorPanelExampleCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that creates the operator panel example project and, if
		 * required, a new general project directory.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			loadExistingModel(AF3ApplicationActivator.PLUGIN_ID,
					"test-data/operator_panel_tutorial.af3_23", "OperatorPanel");
			return null;
		}
	}

	/** Handles the command action to import an existing AF3-Project. */
	public static final class ImportAF3ProjectCommandHandler extends AbstractHandler {

		/** Execute definition that imports an existing AF3-Project. */
		@Override
		public Object execute(ExecutionEvent event) {
			// Open file selection dialog to select the file to be imported.
			String importFile = fileSelectionDialog();

			if(importFile != null) {
				// Copy selected file into workspace.
				importUISelectedFile(createFileURI(importFile));
			}

			return null;
		}
	}

	/** Handles the command action to export an AF3-Project. */
	public static final class ExportAF3ProjectCommandHandler extends AbstractHandler {

		/**
		 * Execute definition that exports an AF3-Project to a user defined
		 * location in the file system.
		 */
		@Override
		public Object execute(ExecutionEvent event) {
			FileProject selectedProject = getSelectedFileProject();

			if(selectedProject != null) {
				// Open file selection dialog to select the target file
				// location.
				String exportFileLocation = fileSelectionDialog(selectedProject.getName());

				// If the user doesn't abort the export during file selection.
				if(exportFileLocation != null) {
					// Use copy to avoid error 'Cannot modify resource set
					// without a write transaction'.
					exportProject(copy(selectedProject), selectedProject.eResource().getURI(),
							createFileURI(exportFileLocation));
				}
			}
			return null;
		}
	}

	/** Show error log action. */
	public static final class ShowErrorLogCommandHandler extends AbstractHandler {

		/** Opens the error log. */
		// TODO(#3367): Avoid the necessity of this warning if possible (won't fix).
		@Override
		public Object execute(ExecutionEvent event) {
			openErrorLogBrowser(Display.getCurrent().getActiveShell());
			return null;
		}
	}

	/** Show error log action. */
	public static final class ShowModelStorageErrorDialogCommandHandler extends AbstractHandler {

		/** Opens the model storage error dialog. */
		@Override
		public Object execute(ExecutionEvent event) {
			List<ModelStorageError> errors =
					IPersistencyService.getInstance().getAllStorageErrors();
			String pid = AF3ApplicationActivator.PLUGIN_ID;
			Status status;
			if(errors.isEmpty()) {
				status = new Status(IStatus.INFO, pid, 0, "No model storage errors found!", null);
			} else {
				MultiStatus info = new MultiStatus(pid, 0, "Model storage errors found!", null);
				for(ModelStorageError error : errors) {
					String message = error.getMessage();
					if(error.getCause() != null) {
						Exception cause = error.getCause();
						message += " (caused by \"" + cause.getClass().getName() + ": " +
								cause.getMessage() + "\")";
					}
					info.add(new Status(IStatus.ERROR, pid, 1, message, null));
				}
				status = info;
			}
			openError(Display.getCurrent().getActiveShell(), "Model Storage Errors", null, status);
			return null;
		}
	}

	/** Refresh the workspace */
	public static final class RefreshWorkspaceCommandHandler extends AbstractHandler {

		/** Refreshes the workspace. */
		@Override
		public Object execute(ExecutionEvent event) {
			refreshWorkspace();
			return null;
		}
	}
}
